/* $Header: /home/felipe/rmarsa/source/rnpl/RCS/librnpl.c,v 1.2 1996/08/06 20:17:37 rmarsa Exp rmarsa $ */
/* Support routines for RNPL generated programs */
/* copyright (c) 1994-1996 by Robert Marsa */

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <string.h>
#include <math.h>
#include <signal.h>
#include <librnpl.h>
#include <mfhdf.h>
#include <stdarg.h>
#include <errno.h>
#include <sys/stat.h>

#define IVEC_SIZE 41

BBH_I rvsxynt(BBH_C *n, BBH_D t, BBH_D *r, 
							BBH_D *u, BBH_I N)
{
#ifdef HAVE_LIBVS
	vsxynt(n,t,r,u,N);
#endif
}

BBH_D rdvprod(BBH_D *r, BBH_I len)
{
	BBH_D d=1.0;
	BBH_I i;
	
	if(r){
		for(i=0;i<len;i++)
			d*=r[i];
	}
	return d;
}

BBH_I rivprod(BBH_I *r, BBH_I len)
{
	BBH_I d=1.0;
	BBH_I i;
	
	if(r)
		for(i=0;i<len;i++)
			d*=r[i];
	return d;
}

BBH_V rdvramp(BBH_D *r, BBH_I len, BBH_D min, BBH_D inc)
{
	BBH_D v;
	BBH_I i;

	if(r)
		for(v=min,i=0;i<len;i++,v+=inc)
			r[i]=v;
}

BBH_V rdvaddc(BBH_D *r, BBH_D c1, BBH_D *s1, BBH_D c2, BBH_D *s2, BBH_I len)
{
	BBH_I i;

	if(r && s1 && s2 && r!=s1 && r!=s2)
		for(i=0;i<len;i++)
			r[i]=c1*s1[i] + c2*s2[i];
}

/* returns the l2norm of a vector */
BBH_D l2norm(BBH_I N, BBH_D *vec)
{
  BBH_I i;
  BBH_D n;

  for(n=0.0,i=0;i<N;i++)
    n+=vec[i]*vec[i];
  return sqrt(n/N);
}

/* returns a vector of size doubles named name;
   prints error message and exits if not enough memory */
BBH_D * vec_alloc_n(BBH_I size, BBH_C * name)
{
  BBH_D *q;

  q=(BBH_D *)malloc(sizeof(BBH_D)*size);
  if(q==NULL){
    fprintf(stderr,"Out of memory error in vec_alloc_n!\n");
    fprintf(stderr,"Unable to malloc memory for %s.\n",name);
    exit(0);
  }
  return q;
}

/* returns a vector of size BBH_Is named name;
   prints error message and exits if not enough memory */
BBH_I * ivec_alloc_n(BBH_I size, BBH_C * name)
{
  BBH_I *q;

  q=(BBH_I *)malloc(sizeof(BBH_I)*size);
  if(q==NULL){
    fprintf(stderr,"Out of memory error in ivec_alloc_n!\n");
    fprintf(stderr,"Unable to malloc memory for %s.\n",name);
    exit(0);
  }
  return q;
}

BBH_V gff_output(BBH_I *ptrs, BBH_D *q, BBH_C **fnames, BBH_I ngfls, 
								BBH_C **cnames, BBH_I ncnames, BBH_D t, BBH_I ser, BBH_I fout,
                BBH_I nout_f, BBH_I *lout_f, BBH_I *gf_st, BBH_I *gf_ln)
{
  BBH_I i,j,k;
	BBH_D *crds,*c;
  latcb *l;

  for(i=0;i<nout_f;i++){
    if(gf_ln[lout_f[i]]==1)
      j=gf_st[lout_f[i]];
    else j=gf_st[lout_f[i]]+1;
    l=(latcb *)(ptrs + ptrs[ngfls+j]-1);
    if(ser && (l->rank == 1)){
      rvsxynt(fnames[lout_f[i]],t,&q[l->cptr[0]-1],&q[ptrs[j]-1],l->shape[0]);
    }
    if(fout){
    	gft_set_d_type(DFNT_FLOAT64);
    	crds=vec_alloc(rivprod(l->shape,l->rank));
    	for(c=crds,k=0;k<l->rank;k++){
    		rdvcpy(c,&q[l->cptr[k]-1],l->shape[k]);
    		c+=l->shape[k];
    	}
			gft_write_full(fnames[lout_f[i]],t,l->shape,cnames+l->cs*(MAXRANK+1)+1,
											l->rank,crds,&q[ptrs[j]-1]);
			free(crds);
    }
  }
}

BBH_V gf_output(gfunc_type *gfuncs, BBH_C **fnames, BBH_C ***cnames,
               BBH_D t, BBH_I ser, BBH_I fout,
               BBH_I nout_f, BBH_I *lout_f,
               BBH_I *gf_st, BBH_I *gf_ln)
{
  BBH_I i,j,k;
	BBH_D *crds,*c;
	
  for(i=0;i<nout_f;i++){
    if(gf_ln[lout_f[i]]==1)
      j=gf_st[lout_f[i]];
    else j=gf_st[lout_f[i]]+1;
    if(ser && (gfuncs[j].lat->rank == 1) && gfuncs[j].type == DFNT_FLOAT64){
      rvsxynt(fnames[lout_f[i]],t,gfuncs[j].lat->coords[0],gfuncs[j].val,
             gfuncs[j].lat->shape[0]);
    }
    if(fout){
    	gft_set_d_type(gfuncs[j].type);
    	crds=vec_alloc(rivprod(gfuncs[j].lat->shape,gfuncs[j].lat->rank));
    	for(c=crds,k=0;k<gfuncs[j].lat->rank;k++){
    		rdvcpy(c,gfuncs[j].lat->coords[k],gfuncs[j].lat->shape[k]);
    		c+=gfuncs[j].lat->shape[k];
    	}
			gft_write_full(fnames[lout_f[i]],t,gfuncs[j].lat->shape,&cnames[gfuncs[j].lat->cs][1],
											gfuncs[j].lat->rank,crds,gfuncs[j].val);
			free(crds);
		}
  }
}

BBH_I encode(BBH_I *lar, BBH_I *ar, BBH_I nar)
{
  BBH_I i,j;

  for(i=0,j=0;i<nar;i++)
    if(ar[i]==1)
      lar[j++]=i;
  return j;
}

/* support for initial data fragment */
static BBH_C id_fname[100];
static BBH_I id_rank;
static BBH_I id_shp[3];

BBH_V rnpl_id_set_file(const BBH_C *fn)
{
	strcpy(id_fname,fn);
}

BBH_V rnpl_id_set_rank(BBH_I r)
{
	id_rank=r;
}

BBH_V rnpl_id_set_shape(BBH_I s1, BBH_I s2, BBH_I s3)
{
	id_shp[0]=s1;
	id_shp[1]=s2;
	id_shp[2]=s3;
}

BBH_V rnpl_id_write(const BBH_C *dsn, BBH_D *data)
{
	gft_write_idata(id_fname,dsn,id_shp,id_rank,data);
}

BBH_V rnpl_id_end()
{
	BBH_D td;
	BBH_I ti;
	
	td=0.0;
	ti=0;
	gft_write_id_float_p(id_fname,"start_t",&td,1);
	gft_write_id_int_p(id_fname,"s_step",&ti,1);
	gft_close(id_fname);
}

/* Interrupt handling for FORTRAN ... */
/* These routines are written by Matthew W. Choptuik */
/* Copyright (c) 1995 by Matthew W. Choptuik */

static BBH_I Interrupt_set = 0;

BBH_V sig_onintr(BBH_I foo) {
   Interrupt_set = 1;
   signal(SIGINT,sig_onintr);
}
	
BBH_V set_interrupt(BBH_V) {
	Interrupt_set = 0;
	signal(SIGINT,sig_onintr);
}

BBH_V unset_interrupt(BBH_V) {
	Interrupt_set = 0;
   signal(SIGINT,SIG_DFL);
}

BBH_V reset_interrupt(BBH_V) {
	Interrupt_set = 0;
}

BBH_I query_interrupt(BBH_V) {
	return Interrupt_set;
}

